# CompressibleTurbulenceGenerator

pyTurbulence is a Python package for generating synthetic compressible turbulent fields for Direct Numerical Simulation (DNS) based on the method of Ristorcelli and Blaisdell. This package provides tools to generate velocity, pressure, and thermodynamic fields, as well as to compute and plot the energy spectrum of the generated turbulence.

TODO:
- Add dilatational velocity fluctuations

![alt text](doc/spectrum.png) ![alt text](doc/u.png)

## Features

- Generate solenoidal velocity fields using Kraichnan's method, improved for mass conservation by Saad.
- Compute incompressible pressure fluctuations using the method of Ristorcelli & Blaisdell.
- Compute thermodynamic fields (density, pressure, temperature) from the generated velocity and pressure fields.
- Plot the energy spectrum of the generated turbulence.

## Installation

To install the package, navigate to the root directory of the project and run:

```sh
pip3 install -- upgrade pip
pip3 install -e .
```

## Usage

To use the code from the `example.py` script, follow these steps:

1. **Navigate to the Project Directory**: Open a terminal and navigate to the root directory of the project.
2. **Run the Example Script**: Execute the `example.py` script to generate synthetic turbulence data and save the results.

    ```sh
    python pyTurbulence/example.py
    ```

3. **View the Results**: The results will be saved in the `results` directory. This includes the generated data files (`u.dat`, `v.dat`, `w.dat`, `p.dat`, `T.dat`), a log file (`log.txt`), a plot of the energy spectrum (`spectrum.png`) and the data of the spectrum (`spectrum.dat`).

## License

This project is licensed under the MIT License. See the LICENSE file for details.

## Acknowledgments

[1] Kraichnan R. H., “Diffusion by a Random Velocity Field,” Physics of Fluids, Vol. 13, No. 1, 1970, Paper 22. doi: https://doi.org/10.1063/1.1692799

[2] Tony Saad, Derek Cline, Rob Stoll, and James C. Sutherland. “Scalable Tools for Generating Synthetic Isotropic Turbulence with Arbitrary Spectra”. http://dx.doi.org/10.2514/1.J055230. (available online: Aug 25, 2016).

[3] Ristorcelli, J. R., & Blaisdell, G. A. (1997). Consistent initial conditions for the DNS of compressible turbulence. Journal of Computational Physics, 140(2), 427-466.