import numpy as np
from pyTurbulence.spectrum import energy_spectrum, compute_tke_spectrum, plot_spectrum
from pyTurbulence.syntheticTurbulence import compute_solenoidal_velocities
import warnings

def test_solenoidal_velocity():
    k = np.linspace(0,1024,1024)
    TKE = 0.5
    urms = np.sqrt(2. * TKE / 3.)
    k0 = 4.0
    name = "PassotPouquet"

    # Compute exact value from the spectrum
    spectrum = energy_spectrum(name, k, urms, k0)
    TKE_spectrum = np.trapz(spectrum, k)
    idxMax = np.argmax(spectrum)
    k0_spectrum = k[idxMax]

    # Generate solenoidal field
    nbModes = 2048
    domain_size = (2. * np.pi, 2. * np.pi, 2. * np.pi)
    domain_resolution = (32,32,32)

    u,v,w,wmax = compute_solenoidal_velocities(name,nbModes, urms, k0, domain_size, domain_resolution)

    # Compute TKE from generated field
    TKE_field = np.mean(0.5*(u.reshape(-1)**2 + v.reshape(-1)**2 + w.reshape(-1)**2))

    # Generate spectrum from solenoidal field
    knyquist, wave_numbers, E_k = compute_tke_spectrum(u, v, w, domain_size[0], domain_size[1], domain_size[2])
    idxMax = np.argmax(E_k)
    k0_field = wave_numbers[idxMax]

    assert np.isclose(TKE_field, TKE_spectrum, rtol=0.05), "Test failed: Generated field does not match expected TKE"
    assert np.isclose(k0_field, k0_spectrum, atol=1.1), "Test failed: Generated field does not conserve TKE"


test_solenoidal_velocity()